//
//  YNDropDownMenu.swift
//  YNDropDownMenu
//
//  Created by YiSeungyoun on 2017. 2. 18..
//  Copyright © 2017년 SeungyounYi. All rights reserved.
//

import UIKit

open class YNDropDownMenu: UIView, YNDropDownDelegate {
    fileprivate var opened: Bool = false
    fileprivate var openedView = UIView()
    fileprivate var openedArrowView = UIImageView()
    fileprivate var openedYNDropDownButton = YNDropDownButton()
    
    fileprivate var dropDownButtons: [YNDropDownButton]?
    fileprivate var menuHeight: CGFloat = 0.0
    fileprivate var numberOfMenu: Int = 0
    
    fileprivate var buttonImages: YNImages?
    fileprivate var buttonlabelFontColors: YNFontColor?
    fileprivate var buttonlabelFonts: YNFont?
    
    fileprivate var _dropDownViews: [UIView]?
    fileprivate var dropDownViews: [UIView]? {
        get {
            return self._dropDownViews
        }
        set {
            guard let _dropDownViews = newValue else { return }
            for view in _dropDownViews {
                if let v = view as? YNDropDownView {
                    v.delegate = self
                }
            }
            self._dropDownViews = newValue
        }
    }
    
    fileprivate var alwaysOnIndex: Int?

    open var blurEffectView: UIView? {
        didSet {
            self.changeBlurEffectView()
        }
    }
    
    open var blurEffectViewAlpha:CGFloat = 1.0
    open var blurEffectStyle:UIBlurEffectStyle = .dark
    
    open var dropDownViewTitles: [String]?

    open var backgroundBlurEnabled = true
    
    open var showMenuDuration = 0.5
    open var hideMenuDuration = 0.3
    
    open var showMenuSpringVelocity:CGFloat = 0.5
    open var showMenuSpringWithDamping:CGFloat = 0.8
    
    open var hideMenuSpringVelocity:CGFloat = 0.9
    open var hideMenuSpringWithDamping:CGFloat = 0.8
    
    public init(frame: CGRect, dropDownViews: [UIView], dropDownViewTitles: [String]) {
        super.init(frame: frame)
        
        if dropDownViews.count != dropDownViewTitles.count {
            fatalError("Please make dropDownViews count same with dropDownViewsTitles count")
        } else {
            numberOfMenu = dropDownViews.count
        }
        
        self.dropDownViews = dropDownViews
        
        self.dropDownViewTitles = dropDownViewTitles
        self.menuHeight = self.frame.height
        
        self.initViews()
    }
    
    @available(*, deprecated, message: "use init(frame: CGRect, dropDownViews: [UIView], dropDownViewTitles: [String]) instead")
    public init(frame: CGRect, YNDropDownViews: [YNDropDownView], dropDownViewTitles: [String]) {
        super.init(frame: frame)

        if YNDropDownViews.count != dropDownViewTitles.count {
            fatalError("Please make dropDownViews count same with dropDownViewsTitles count")
        } else {
            numberOfMenu = YNDropDownViews.count
        }

        self._dropDownViews = YNDropDownViews
        self.dropDownViewTitles = dropDownViewTitles
        self.menuHeight = self.frame.height

        self.initViews()
    }
    
    required public init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // Use this function when your menu button image is all same
    open func setImageWhen(normal: UIImage?, selected: UIImage?, disabled: UIImage?) {
        self.buttonImages = YNImages.init(normal: normal, selected: selected, disabled: disabled)
        
        for i in 0..<numberOfMenu {
            dropDownButtons?[i].buttonImages = self.buttonImages
        }
    }
    
    open func setLabelColorWhen(normal: UIColor, selected: UIColor, disabled: UIColor) {
        self.buttonlabelFontColors = YNFontColor.init(normal: normal, selected: selected, disabled: disabled)
        
        for i in 0..<numberOfMenu {
            dropDownButtons?[i].labelFontColors = self.buttonlabelFontColors
        }
    }
    
    open func setLabelFontWhen(normal: UIFont, selected: UIFont, disabled: UIFont) {
        self.buttonlabelFonts = YNFont.init(normal: normal, selected: selected, disabled: disabled)
        
        for i in 0..<numberOfMenu {
            dropDownButtons?[i].labelFonts = self.buttonlabelFonts
        }

    }
    
    open func alwaysSelectedAt(index: Int) {
        if index > numberOfMenu {
            fatalError("index should be smaller than menu count")
        }
        self.alwaysOnIndex = index
        
        for subview in self.subviews {
            if subview.tag == index {
                if subview.isKind(of: YNDropDownButton.self) {
                    let _subview = subview as! YNDropDownButton
                    _subview.buttonLabel.textColor = self.buttonlabelFontColors?.selected
                    _subview.buttonLabel.font = self.buttonlabelFonts?.selected

                }
            }
        }
    }
    
    open func disabledMenuAt(index: Int) {
        if index > numberOfMenu {
            fatalError("index should be smaller than menu count")
        }
        for subview in self.subviews {
            if subview.tag == index {
                if subview.isKind(of: YNDropDownButton.self) {
                    let _subview = subview as! YNDropDownButton
                    _subview.disabled()
                }
            }
        }
    }
    
    open func enabledMenuAt(index: Int) {
        if index > numberOfMenu {
            fatalError("index should be smaller than menu count")
        }
        for subview in self.subviews {
            if subview.tag == index {
                if subview.isKind(of: YNDropDownButton.self) {
                    let _subview = subview as! YNDropDownButton
                    _subview.enabled()
                }
            }
        }
    }
    
    open func hideMenu() {
        if opened {
            hideMenu(yNDropDownButton: openedYNDropDownButton, arrowView: openedArrowView, dropDownMenu: openedView, didComplete: nil)
            opened = !opened
        }
    }
    
    open func changeMenuTitleAt(index: Int, title: String) {
        for subview in self.subviews {
            if subview.tag == index {
                if subview.isKind(of: YNDropDownButton.self) {
                    let _subview = subview as! YNDropDownButton
                    _subview.buttonLabel.text = title
                }
            }
        }

    }

    open func showAndHideMenuAt(index: Int) {
        if index > numberOfMenu {
            fatalError("index should be smaller than menu count")
        }
        
        var dropDownView = UIView()
        var buttonImageView = UIImageView()
        var yNDropDownButton = YNDropDownButton()

        for subview in self.subviews {
            if subview.tag == index + 100 {
                dropDownView = subview
            }
            
            if subview.tag == index {
                if subview.isKind(of: YNDropDownButton.self) {
                    let _subview = subview as! YNDropDownButton
                    yNDropDownButton = _subview
                    buttonImageView = _subview.buttonImageView
                }
            }
        }
        
        if openedView != dropDownView && opened {
            hideMenu(yNDropDownButton: openedYNDropDownButton, arrowView: openedArrowView, dropDownMenu: openedView, didComplete: {
                self.showMenu(yNDropDownButton: yNDropDownButton, arrowView: buttonImageView, dropDownMenu: dropDownView, didComplete: nil)
            })
            openedYNDropDownButton = yNDropDownButton
            openedArrowView = buttonImageView
            openedView = dropDownView
            return
        }
        
        openedYNDropDownButton = yNDropDownButton
        openedArrowView = buttonImageView
        openedView = dropDownView
        
        if !opened {
            showMenu(yNDropDownButton: yNDropDownButton, arrowView: buttonImageView, dropDownMenu: dropDownView, didComplete: nil)
        } else {
            hideMenu(yNDropDownButton: yNDropDownButton, arrowView: buttonImageView, dropDownMenu: dropDownView, didComplete: nil)

        }
        opened = !opened

    }
    @objc private func menuClicked(_ sender: YNDropDownButton) {
        self.showAndHideMenuAt(index: sender.tag)
    }
    
    @objc private func blurEffectViewClicked(_ sender: UITapGestureRecognizer) {
        self.hideMenu()
    }

    
    private func showMenu(yNDropDownButton: YNDropDownButton, arrowView: UIImageView, dropDownMenu: UIView, didComplete: (()-> Void)?) {
        dropDownMenu.isHidden = false
        
        if let v = dropDownMenu as? YNDropDownView {
            v.dropDownViewOpened()
        }
        
        if self.backgroundBlurEnabled, let _blurEffectView = blurEffectView {
            self.superview?.addSubview(_blurEffectView)
            self.superview?.insertSubview(_blurEffectView, belowSubview: self)
        }
        UIView.animate(
            withDuration: self.showMenuDuration,
            delay: 0,
            usingSpringWithDamping: self.showMenuSpringWithDamping,
            initialSpringVelocity: self.showMenuSpringVelocity,
            options: [],
            animations: {
                dropDownMenu.frame.origin.y = CGFloat(self.menuHeight)
                if self.backgroundBlurEnabled {
                    self.blurEffectView?.alpha = self.blurEffectViewAlpha
                }
                
                self.frame = CGRect(x: 0, y: self.frame.origin.y, width: self.frame.width, height: dropDownMenu.frame.height + CGFloat(self.menuHeight))

                arrowView.layer.transform = CATransform3DMakeRotation(CGFloat(M_PI), 1.0, 0.0, 0.0)
                arrowView.image = self.buttonImages?.selected
                yNDropDownButton.buttonLabel.textColor = self.buttonlabelFontColors?.selected
                yNDropDownButton.buttonLabel.font = self.buttonlabelFonts?.selected
                
        }, completion: { (completion) in
            guard let block = didComplete else { return }
            block()
        })
    }
    
    private func hideMenu(yNDropDownButton: YNDropDownButton,arrowView: UIImageView, dropDownMenu: UIView, didComplete: (()-> Void)?) {
        dropDownMenu.isHidden = true
        
        if let v = dropDownMenu as? YNDropDownView {
            v.dropDownViewClosed()
        }

        UIView.animate(
            withDuration: self.hideMenuDuration,
            delay: 0,
            usingSpringWithDamping: self.hideMenuSpringWithDamping,
            initialSpringVelocity: self.hideMenuSpringVelocity,
            options: [],
            animations: {
                dropDownMenu.frame.origin.y = CGFloat(self.menuHeight)
                if self.backgroundBlurEnabled {
                    self.blurEffectView?.alpha = 0
                }
                self.frame = CGRect(x: 0.0, y: self.frame.origin.y, width: self.frame.width, height: CGFloat(self.menuHeight))

                arrowView.layer.transform = CATransform3DMakeRotation(CGFloat(M_PI), 0.0, 0.0, 0.0);
                arrowView.image = self.buttonImages?.normal
                
                if self.alwaysOnIndex != yNDropDownButton.tag {
                    yNDropDownButton.buttonLabel.textColor = self.buttonlabelFontColors?.normal
                    yNDropDownButton.buttonLabel.font = self.buttonlabelFonts?.normal
                }

        }, completion: { (completion) in
            if self.backgroundBlurEnabled {
                self.blurEffectView?.removeFromSuperview()
            }
            guard let block = didComplete else { return }
            block()
        })
    }
    
    
    private func changeBlurEffectView() {
        self.blurEffectView?.frame = CGRect(x: self.frame.origin.x, y: self.frame.origin.y, width: self.frame.width, height: UIScreen.main.bounds.size.height - self.frame.origin.y)
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(blurEffectViewClicked(_:)))
        self.blurEffectView?.addGestureRecognizer(tapGesture)
        self.blurEffectView?.alpha = 0

    }
    
    
    
    private func initViews() {
        self.clipsToBounds = true
        
        self.backgroundColor = UIColor.white
        self.dropDownButtons = [YNDropDownButton]()
        
        let eachWidth = self.bounds.size.width / CGFloat(numberOfMenu)
        
        for i in 0..<numberOfMenu {
            // Setup button
            let button = YNDropDownButton(frame: CGRect(x: eachWidth * CGFloat(i), y: 0.0, width: eachWidth, height: CGFloat(menuHeight)), buttonLabelText: dropDownViewTitles?[i])
            button.tag = i
            button.addTarget(self, action: #selector(menuClicked(_:)), for: .touchUpInside)
            dropDownButtons?.append(button)
            
            self.addSubview(button)
            
            // Setup Views
            if let _dropDownMenu = dropDownViews?[i] {
                _dropDownMenu.frame.size = CGSize(width: self.bounds.size.width, height: _dropDownMenu.frame.height)
                _dropDownMenu.frame.origin.y = -_dropDownMenu.frame.height + CGFloat(menuHeight)
                _dropDownMenu.tag = i + 100
                _dropDownMenu.isHidden = true
                
                self.addSubview(_dropDownMenu)
                self.sendSubview(toBack: _dropDownMenu)
            }
            
        }
        
        let blurEffect = UIBlurEffect(style: blurEffectStyle)
        self.blurEffectView = UIVisualEffectView(effect: blurEffect)
        self.blurEffectView?.alpha = 0
        
        self.blurEffectView?.frame = CGRect(x: self.frame.origin.x, y: self.frame.origin.y, width: self.frame.width, height: UIScreen.main.bounds.size.height - self.frame.origin.y)
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(blurEffectViewClicked(_:)))
        self.blurEffectView?.addGestureRecognizer(tapGesture)
    }
}
